@@ -1,18 +1,18 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# :Id: $Id: latex2mathml.py 7668 2013-06-04 12:46:30Z milde $
+# :Id: $Id: latex2mathml.py 8675 2021-04-08 16:35:54Z milde $
 # :Copyright: © 2010 Günter Milde.
 #             Based on rst2mathml.py from the latex_math sandbox project
 #             © 2005 Jens Jørgen Mortensen
 # :License: Released under the terms of the `2-Clause BSD license`_, in short:
-# 
+#
 #    Copying and distribution of this file, with or without modification,
 #    are permitted in any medium without royalty provided the copyright
 #    notice and this notice are preserved.
 #    This file is offered as-is, without any warranty.
-# 
-# .. _2-Clause BSD license: http://www.spdx.org/licenses/BSD-2-Clause
+#
+# .. _2-Clause BSD license: https://opensource.org/licenses/BSD-2-Clause
 
 
 """Convert LaTex math code into presentational MathML"""
@@ -22,26 +22,26 @@
 import docutils.utils.math.tex2unichar as tex2unichar
 
 #        TeX      spacing    combining
-over = {'acute':    '\u00B4', # u'\u0301',
-        'bar':      '\u00AF', # u'\u0304',
-        'breve':    '\u02D8', # u'\u0306',
-        'check':    '\u02C7', # u'\u030C',
-        'dot':      '\u02D9', # u'\u0307',
-        'ddot':     '\u00A8', # u'\u0308',
-        'dddot':               '\u20DB',
-        'grave':    '`',      # u'\u0300',
-        'hat':      '^',      # u'\u0302',
-        'mathring': '\u02DA', # u'\u030A',
-        'overleftrightarrow':  '\u20e1',
+over = {'acute':    u'\u00B4', # u'\u0301',
+        'bar':      u'\u00AF', # u'\u0304',
+        'breve':    u'\u02D8', # u'\u0306',
+        'check':    u'\u02C7', # u'\u030C',
+        'dot':      u'\u02D9', # u'\u0307',
+        'ddot':     u'\u00A8', # u'\u0308',
+        'dddot':               u'\u20DB',
+        'grave':    u'`',      # u'\u0300',
+        'hat':      u'^',      # u'\u0302',
+        'mathring': u'\u02DA', # u'\u030A',
+        'overleftrightarrow':  u'\u20e1',
         # 'overline':        # u'\u0305',
-        'tilde':    '\u02DC', # u'\u0303',
-        'vec':               '\u20D7'}
+        'tilde':    u'\u02DC', # u'\u0303',
+        'vec':               u'\u20D7'}
 
 Greek = { # Capital Greek letters: (upright in TeX style)
-    'Phi':'\u03a6', 'Xi':'\u039e', 'Sigma':'\u03a3',
-    'Psi':'\u03a8', 'Delta':'\u0394', 'Theta':'\u0398',
-    'Upsilon':'\u03d2', 'Pi':'\u03a0', 'Omega':'\u03a9',
-    'Gamma':'\u0393', 'Lambda':'\u039b'}
+    'Phi':u'\u03a6', 'Xi':u'\u039e', 'Sigma':u'\u03a3',
+    'Psi':u'\u03a8', 'Delta':u'\u0394', 'Theta':u'\u0398',
+    'Upsilon':u'\u03d2', 'Pi':u'\u03a0', 'Omega':u'\u03a9',
+    'Gamma':u'\u0393', 'Lambda':u'\u039b'}
 
 letters = tex2unichar.mathalpha
 
@@ -67,95 +67,95 @@ functions = ['arccos', 'arcsin', 'arctan', 'arg', 'cos',  'cosh',
 
 
 mathbb = {
-          'A': '\U0001D538',
-          'B': '\U0001D539',
-          'C': '\u2102',
-          'D': '\U0001D53B',
-          'E': '\U0001D53C',
-          'F': '\U0001D53D',
-          'G': '\U0001D53E',
-          'H': '\u210D',
-          'I': '\U0001D540',
-          'J': '\U0001D541',
-          'K': '\U0001D542',
-          'L': '\U0001D543',
-          'M': '\U0001D544',
-          'N': '\u2115',
-          'O': '\U0001D546',
-          'P': '\u2119',
-          'Q': '\u211A',
-          'R': '\u211D',
-          'S': '\U0001D54A',
-          'T': '\U0001D54B',
-          'U': '\U0001D54C',
-          'V': '\U0001D54D',
-          'W': '\U0001D54E',
-          'X': '\U0001D54F',
-          'Y': '\U0001D550',
-          'Z': '\u2124',
+          'A': u'\U0001D538',
+          'B': u'\U0001D539',
+          'C': u'\u2102',
+          'D': u'\U0001D53B',
+          'E': u'\U0001D53C',
+          'F': u'\U0001D53D',
+          'G': u'\U0001D53E',
+          'H': u'\u210D',
+          'I': u'\U0001D540',
+          'J': u'\U0001D541',
+          'K': u'\U0001D542',
+          'L': u'\U0001D543',
+          'M': u'\U0001D544',
+          'N': u'\u2115',
+          'O': u'\U0001D546',
+          'P': u'\u2119',
+          'Q': u'\u211A',
+          'R': u'\u211D',
+          'S': u'\U0001D54A',
+          'T': u'\U0001D54B',
+          'U': u'\U0001D54C',
+          'V': u'\U0001D54D',
+          'W': u'\U0001D54E',
+          'X': u'\U0001D54F',
+          'Y': u'\U0001D550',
+          'Z': u'\u2124',
          }
 
 mathscr = {
-           'A': '\U0001D49C',
-           'B': '\u212C',     # bernoulli function
-           'C': '\U0001D49E',
-           'D': '\U0001D49F',
-           'E': '\u2130',
-           'F': '\u2131',
-           'G': '\U0001D4A2',
-           'H': '\u210B',     # hamiltonian
-           'I': '\u2110',
-           'J': '\U0001D4A5',
-           'K': '\U0001D4A6',
-           'L': '\u2112',     # lagrangian
-           'M': '\u2133',     # physics m-matrix
-           'N': '\U0001D4A9',
-           'O': '\U0001D4AA',
-           'P': '\U0001D4AB',
-           'Q': '\U0001D4AC',
-           'R': '\u211B',
-           'S': '\U0001D4AE',
-           'T': '\U0001D4AF',
-           'U': '\U0001D4B0',
-           'V': '\U0001D4B1',
-           'W': '\U0001D4B2',
-           'X': '\U0001D4B3',
-           'Y': '\U0001D4B4',
-           'Z': '\U0001D4B5',
-           'a': '\U0001D4B6',
-           'b': '\U0001D4B7',
-           'c': '\U0001D4B8',
-           'd': '\U0001D4B9',
-           'e': '\u212F',
-           'f': '\U0001D4BB',
-           'g': '\u210A',
-           'h': '\U0001D4BD',
-           'i': '\U0001D4BE',
-           'j': '\U0001D4BF',
-           'k': '\U0001D4C0',
-           'l': '\U0001D4C1',
-           'm': '\U0001D4C2',
-           'n': '\U0001D4C3',
-           'o': '\u2134',     # order of
-           'p': '\U0001D4C5',
-           'q': '\U0001D4C6',
-           'r': '\U0001D4C7',
-           's': '\U0001D4C8',
-           't': '\U0001D4C9',
-           'u': '\U0001D4CA',
-           'v': '\U0001D4CB',
-           'w': '\U0001D4CC',
-           'x': '\U0001D4CD',
-           'y': '\U0001D4CE',
-           'z': '\U0001D4CF',
+           'A': u'\U0001D49C',
+           'B': u'\u212C',     # bernoulli function
+           'C': u'\U0001D49E',
+           'D': u'\U0001D49F',
+           'E': u'\u2130',
+           'F': u'\u2131',
+           'G': u'\U0001D4A2',
+           'H': u'\u210B',     # hamiltonian
+           'I': u'\u2110',
+           'J': u'\U0001D4A5',
+           'K': u'\U0001D4A6',
+           'L': u'\u2112',     # lagrangian
+           'M': u'\u2133',     # physics m-matrix
+           'N': u'\U0001D4A9',
+           'O': u'\U0001D4AA',
+           'P': u'\U0001D4AB',
+           'Q': u'\U0001D4AC',
+           'R': u'\u211B',
+           'S': u'\U0001D4AE',
+           'T': u'\U0001D4AF',
+           'U': u'\U0001D4B0',
+           'V': u'\U0001D4B1',
+           'W': u'\U0001D4B2',
+           'X': u'\U0001D4B3',
+           'Y': u'\U0001D4B4',
+           'Z': u'\U0001D4B5',
+           'a': u'\U0001D4B6',
+           'b': u'\U0001D4B7',
+           'c': u'\U0001D4B8',
+           'd': u'\U0001D4B9',
+           'e': u'\u212F',
+           'f': u'\U0001D4BB',
+           'g': u'\u210A',
+           'h': u'\U0001D4BD',
+           'i': u'\U0001D4BE',
+           'j': u'\U0001D4BF',
+           'k': u'\U0001D4C0',
+           'l': u'\U0001D4C1',
+           'm': u'\U0001D4C2',
+           'n': u'\U0001D4C3',
+           'o': u'\u2134',     # order of
+           'p': u'\U0001D4C5',
+           'q': u'\U0001D4C6',
+           'r': u'\U0001D4C7',
+           's': u'\U0001D4C8',
+           't': u'\U0001D4C9',
+           'u': u'\U0001D4CA',
+           'v': u'\U0001D4CB',
+           'w': u'\U0001D4CC',
+           'x': u'\U0001D4CD',
+           'y': u'\U0001D4CE',
+           'z': u'\U0001D4CF',
           }
 
-negatables = {'=': '\u2260',
-              '\in': '\u2209',
-              '\equiv': '\u2262'}
+negatables = {'=': u'\u2260',
+              r'\in': u'\u2209',
+              r'\equiv': u'\u2262'}
 
 # LaTeX to MathML translation stuff:
-class math:
+class math(object):
     """Base class for MathML elements."""
 
     nchildren = 1000000
@@ -168,7 +168,7 @@ class math:
 
         self.children = []
         if children is not None:
-            if type(children) is list:
+            if isinstance(children, list):
                 for child in children:
                     self.append(child)
             else:
@@ -304,8 +304,8 @@ class msubsup(math):
         return math.xml(self)
 
 class mfenced(math):
-    translation = {'\\{': '{', '\\langle': '\u2329',
-                   '\\}': '}', '\\rangle': '\u232A',
+    translation = {'\\{': '{', '\\langle': u'\u2329',
+                   '\\}': '}', '\\rangle': u'\u232A',
                    '.': ''}
     def __init__(self, par):
         self.openpar = par
@@ -328,7 +328,7 @@ class mstyle(math):
 
     def xml_start(self):
         return ['<mstyle '] + ['%s="%s"' % item
-                               for item in list(self.attrs.items())] + ['>']
+                               for item in self.attrs.items()] + ['>']
 
 class mover(math):
     nchildren = 2
@@ -384,7 +384,6 @@ def parse_latex_math(string, inline=True):
             c2 = string[1]
         else:
             c2 = ''
-##        print n, string, c, c2, node.__class__.__name__
         if c == ' ':
             pass
         elif c == '\\':
@@ -413,7 +412,7 @@ def parse_latex_math(string, inline=True):
                 node = entry
                 skip = 2
             else:
-                raise SyntaxError(r'Syntax error: "%s%s"' % (c, c2))
+                raise SyntaxError(u'Syntax error: "%s%s"' % (c, c2))
         elif c.isalpha():
             node = node.append(mi(c))
         elif c.isdigit():
@@ -454,7 +453,7 @@ def parse_latex_math(string, inline=True):
             node.close().append(entry)
             node = entry
         else:
-            raise SyntaxError(r'Illegal character: "%s"' % c)
+            raise SyntaxError(u'Illegal character: "%s"' % c)
         string = string[skip:]
     return tree
 
@@ -466,8 +465,8 @@ def handle_keyword(name, node, string):
         skip = 1
     if name == 'begin':
         if not string.startswith('{matrix}'):
-            raise SyntaxError('Environment not supported! '
-                              'Supported environment: "matrix".')
+            raise SyntaxError(u'Environment not supported! '
+                              u'Supported environment: "matrix".')
         skip += 8
         entry = mtd()
         table = mtable(mtr(entry))
@@ -475,15 +474,15 @@ def handle_keyword(name, node, string):
         node = entry
     elif name == 'end':
         if not string.startswith('{matrix}'):
-            raise SyntaxError(r'Expected "\end{matrix}"!')
+            raise SyntaxError(u'Expected "\\end{matrix}"!')
         skip += 8
         node = node.close().close().close()
     elif name in ('text', 'mathrm'):
         if string[0] != '{':
-            raise SyntaxError(r'Expected "\text{...}"!')
+            raise SyntaxError(u'Expected "\\text{...}"!')
         i = string.find('}')
         if i == -1:
-            raise SyntaxError(r'Expected "\text{...}"!')
+            raise SyntaxError(u'Expected "\\text{...}"!')
         node = node.append(mtext(string[1:i]))
         skip += i + 1
     elif name == 'sqrt':
@@ -499,7 +498,7 @@ def handle_keyword(name, node, string):
             if string.startswith(par):
                 break
         else:
-            raise SyntaxError('Missing left-brace!')
+            raise SyntaxError(u'Missing left-brace!')
         fenced = mfenced(par)
         node.append(fenced)
         row = mrow()
@@ -511,7 +510,7 @@ def handle_keyword(name, node, string):
             if string.startswith(par):
                 break
         else:
-            raise SyntaxError('Missing right-brace!')
+            raise SyntaxError(u'Missing right-brace!')
         node = node.close()
         node.closepar = par
         node = node.close()
@@ -521,21 +520,21 @@ def handle_keyword(name, node, string):
             if string.startswith(operator):
                 break
         else:
-            raise SyntaxError(r'Expected something to negate: "\not ..."!')
+            raise SyntaxError(u'Expected something to negate: "\\not ..."!')
         node = node.append(mo(negatables[operator]))
         skip += len(operator)
     elif name == 'mathbf':
-        style = mstyle(nchildren=1, fontweight='bold')
+        style = mstyle(nchildren=1, mathvariant='bold')
         node.append(style)
         node = style
     elif name == 'mathbb':
         if string[0] != '{' or not string[1].isupper() or string[2] != '}':
-            raise SyntaxError(r'Expected something like "\mathbb{A}"!')
+            raise SyntaxError(u'Expected something like "\\mathbb{A}"!')
         node = node.append(mi(mathbb[string[1]]))
         skip += 3
     elif name in ('mathscr', 'mathcal'):
         if string[0] != '{' or string[2] != '}':
-            raise SyntaxError(r'Expected something like "\mathscr{A}"!')
+            raise SyntaxError(u'Expected something like "\\mathscr{A}"!')
         node = node.append(mi(mathscr[string[1]]))
         skip += 3
     elif name == 'colon': # "normal" colon, not binary operator
@@ -555,6 +554,15 @@ def handle_keyword(name, node, string):
         node.append(ovr)
         node = ovr
     else:
-        raise SyntaxError('Unknown LaTeX command: ' + name)
+        raise SyntaxError(u'Unknown LaTeX command: ' + name)
 
     return node, skip
+
+def tex2mathml(tex_math, inline=True):
+    """Return string with MathML code corresponding to `tex_math`.
+
+    `inline`=True is for inline math and `inline`=False for displayed math.
+    """
+
+    mathml_tree = parse_latex_math(tex_math, inline=inline)
+    return ''.join(mathml_tree.xml())
